@@ -1,42 +1,22 @@
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+#[cfg(feature = "v1_18")]
 use std::mem::transmute;
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+#[cfg(feature = "v1_18")]
 use glib::object::Cast;
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+#[cfg(feature = "v1_18")]
 use glib::signal::{connect_raw, SignalHandlerId};
-use glib::{object::IsA, translate::*};
+use glib::translate::*;
 use gst::prelude::*;
 
-use crate::auto::{AudioAggregator, AudioAggregatorPad};
+use crate::{ffi, AudioAggregator, AudioAggregatorPad};
 
-pub trait AudioAggregatorExtManual: 'static {
-    #[doc(alias = "gst_audio_aggregator_set_sink_caps")]
-    fn set_sink_caps(&self, pad: &impl IsA<AudioAggregatorPad>, caps: &gst::CapsRef);
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    #[doc(alias = "output-buffer-duration-fraction")]
-    fn output_buffer_duration_fraction(&self) -> gst::Fraction;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    #[doc(alias = "output-buffer-duration-fraction")]
-    fn set_output_buffer_duration_fraction(&self, output_buffer_duration_fraction: gst::Fraction);
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    #[doc(alias = "output-buffer-duration-fraction")]
-    fn connect_output_buffer_duration_fraction_notify<F: Fn(&Self) + Send + Sync + 'static>(
-        &self,
-        f: F,
-    ) -> SignalHandlerId;
-
-    fn current_caps(&self) -> Option<gst::Caps>;
-    fn current_audio_info(&self) -> Option<crate::AudioInfo>;
+mod sealed {
+    pub trait Sealed {}
+    impl<T: super::IsA<super::AudioAggregator>> Sealed for T {}
 }
 
-impl<O: IsA<AudioAggregator>> AudioAggregatorExtManual for O {
+pub trait AudioAggregatorExtManual: sealed::Sealed + IsA<AudioAggregator> + 'static {
+    #[doc(alias = "gst_audio_aggregator_set_sink_caps")]
     fn set_sink_caps(&self, pad: &impl IsA<AudioAggregatorPad>, caps: &gst::CapsRef) {
         unsafe {
             ffi::gst_audio_aggregator_set_sink_caps(
@@ -47,24 +27,27 @@ impl<O: IsA<AudioAggregator>> AudioAggregatorExtManual for O {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
+    #[doc(alias = "output-buffer-duration-fraction")]
     fn output_buffer_duration_fraction(&self) -> gst::Fraction {
-        glib::ObjectExt::property(self.as_ref(), "output-buffer-duration-fraction")
+        ObjectExt::property(self.as_ref(), "output-buffer-duration-fraction")
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
+    #[doc(alias = "output-buffer-duration-fraction")]
     fn set_output_buffer_duration_fraction(&self, output_buffer_duration_fraction: gst::Fraction) {
-        glib::ObjectExt::set_property(
+        ObjectExt::set_property(
             self.as_ref(),
             "output-buffer-duration-fraction",
             output_buffer_duration_fraction,
         )
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
+    #[doc(alias = "output-buffer-duration-fraction")]
     fn connect_output_buffer_duration_fraction_notify<F: Fn(&Self) + Send + Sync + 'static>(
         &self,
         f: F,
@@ -85,7 +68,7 @@ impl<O: IsA<AudioAggregator>> AudioAggregatorExtManual for O {
             connect_raw(
                 self.as_ptr() as *mut _,
                 b"notify::output-buffer-duration-fraction\0".as_ptr() as *const _,
-                Some(transmute::<_, unsafe extern "C" fn()>(
+                Some(transmute::<*const (), unsafe extern "C" fn()>(
                     notify_output_buffer_duration_fraction_trampoline::<Self, F> as *const (),
                 )),
                 Box::into_raw(f),
@@ -106,3 +89,5 @@ impl<O: IsA<AudioAggregator>> AudioAggregatorExtManual for O {
             .and_then(|caps| crate::AudioInfo::from_caps(&caps).ok())
     }
 }
+
+impl<O: IsA<AudioAggregator>> AudioAggregatorExtManual for O {}
