@@ -1,12 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::cmp::Ordering;
-use std::ffi::CStr;
-use std::fmt;
-use std::str;
+use std::{cmp::Ordering, fmt, marker::PhantomData, str};
 
-use glib::translate::{from_glib, from_glib_none, FromGlib, IntoGlib, ToGlibPtr, ToGlibPtrMut};
-use glib::StaticType;
+use crate::ffi;
+use glib::{
+    prelude::*,
+    translate::{from_glib, from_glib_none, FromGlib, IntoGlib, ToGlibPtr, ToGlibPtrMut},
+};
 
 #[derive(PartialEq, Eq, Copy, Clone, Debug, Hash)]
 pub enum AudioEndianness {
@@ -17,8 +17,9 @@ pub enum AudioEndianness {
 
 impl FromGlib<i32> for AudioEndianness {
     #[allow(unused_unsafe)]
+    #[inline]
     unsafe fn from_glib(value: i32) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         match value {
             1234 => Self::LittleEndian,
@@ -31,6 +32,7 @@ impl FromGlib<i32> for AudioEndianness {
 impl IntoGlib for AudioEndianness {
     type GlibType = i32;
 
+    #[inline]
     fn into_glib(self) -> i32 {
         match self {
             Self::LittleEndian => 1234,
@@ -41,52 +43,63 @@ impl IntoGlib for AudioEndianness {
 }
 
 #[doc(alias = "GstAudioFormatInfo")]
+#[derive(Copy, Clone)]
 pub struct AudioFormatInfo(&'static ffi::GstAudioFormatInfo);
 
 impl AudioFormatInfo {
+    #[inline]
     pub fn from_format(format: crate::AudioFormat) -> Self {
         assert_initialized_main_thread!();
 
         unsafe {
             let info = ffi::gst_audio_format_get_info(format.into_glib());
-            assert!(!info.is_null());
+            debug_assert!(!info.is_null());
 
             Self(&*info)
         }
     }
 
+    #[inline]
     pub fn format(&self) -> crate::AudioFormat {
         unsafe { from_glib(self.0.format) }
     }
 
-    pub fn name<'a>(&self) -> &'a str {
-        unsafe { CStr::from_ptr(self.0.name).to_str().unwrap() }
+    #[inline]
+    pub fn name<'a>(&self) -> &'a glib::GStr {
+        unsafe { glib::GStr::from_ptr(self.0.name) }
     }
 
-    pub fn description<'a>(&self) -> &'a str {
-        unsafe { CStr::from_ptr(self.0.description).to_str().unwrap() }
+    #[inline]
+    pub fn description<'a>(&self) -> &'a glib::GStr {
+        unsafe { glib::GStr::from_ptr(self.0.description) }
     }
 
+    #[inline]
     pub fn flags(&self) -> crate::AudioFormatFlags {
         unsafe { from_glib(self.0.flags) }
     }
 
+    #[inline]
     pub fn endianness(&self) -> AudioEndianness {
         unsafe { from_glib(self.0.endianness) }
     }
 
+    #[inline]
     pub fn width(&self) -> u32 {
         self.0.width as u32
     }
 
+    #[inline]
     pub fn depth(&self) -> u32 {
         self.0.depth as u32
     }
 
+    #[inline]
     pub fn unpack_format(&self) -> crate::AudioFormat {
         unsafe { from_glib(self.0.unpack_format) }
     }
 
+    #[inline]
     pub fn silence<'a>(&self) -> &'a [u8] {
         &self.0.silence
     }
@@ -96,12 +109,12 @@ impl AudioFormatInfo {
         let unpack_width = unpack_format.width() as usize;
 
         if unpack_width == 0 || self.0.unpack_func.is_none() {
-            panic!("No unpack format for {:?}", self);
+            panic!("No unpack format for {self:?}");
         }
 
         let self_width = self.width() as usize;
         if self_width == 0 {
-            panic!("No width for {:?}", self);
+            panic!("No width for {self:?}");
         }
 
         if src.len() % (self_width / 8) != 0 {
@@ -130,12 +143,12 @@ impl AudioFormatInfo {
         let unpack_width = unpack_format.width() as usize;
 
         if unpack_width == 0 || self.0.pack_func.is_none() {
-            panic!("No unpack format for {:?}", self);
+            panic!("No unpack format for {self:?}");
         }
 
         let self_width = self.width() as usize;
         if self_width == 0 {
-            panic!("No width for {:?}", self);
+            panic!("No width for {self:?}");
         }
 
         if src.len() % (unpack_width / 8) != 0 {
@@ -183,22 +196,27 @@ impl AudioFormatInfo {
         }
     }
 
+    #[inline]
     pub fn is_float(&self) -> bool {
         self.flags().contains(crate::AudioFormatFlags::FLOAT)
     }
 
+    #[inline]
     pub fn is_integer(&self) -> bool {
         self.flags().contains(crate::AudioFormatFlags::INTEGER)
     }
 
+    #[inline]
     pub fn is_signed(&self) -> bool {
         self.flags().contains(crate::AudioFormatFlags::SIGNED)
     }
 
+    #[inline]
     pub fn is_little_endian(&self) -> bool {
         self.endianness() == AudioEndianness::LittleEndian
     }
 
+    #[inline]
     pub fn is_big_endian(&self) -> bool {
         self.endianness() == AudioEndianness::BigEndian
     }
@@ -208,6 +226,7 @@ unsafe impl Sync for AudioFormatInfo {}
 unsafe impl Send for AudioFormatInfo {}
 
 impl PartialEq for AudioFormatInfo {
+    #[inline]
     fn eq(&self, other: &Self) -> bool {
         self.format() == other.format()
     }
@@ -216,6 +235,7 @@ impl PartialEq for AudioFormatInfo {
 impl Eq for AudioFormatInfo {}
 
 impl PartialOrd for AudioFormatInfo {
+    #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
     }
@@ -305,6 +325,7 @@ impl str::FromStr for crate::AudioFormatInfo {
 }
 
 impl From<crate::AudioFormat> for AudioFormatInfo {
+    #[inline]
     fn from(f: crate::AudioFormat) -> Self {
         skip_assert_initialized!();
         Self::from_format(f)
@@ -312,6 +333,7 @@ impl From<crate::AudioFormat> for AudioFormatInfo {
 }
 
 impl glib::types::StaticType for AudioFormatInfo {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { glib::translate::from_glib(ffi::gst_audio_format_info_get_type()) }
     }
@@ -365,17 +387,29 @@ impl glib::value::ToValueOptional for AudioFormatInfo {
     }
 }
 
+#[doc(hidden)]
+impl From<AudioFormatInfo> for glib::Value {
+    fn from(v: AudioFormatInfo) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[doc(hidden)]
 impl glib::translate::GlibPtrDefault for AudioFormatInfo {
     type GlibType = *mut ffi::GstAudioFormatInfo;
 }
 
 #[doc(hidden)]
-impl<'a> glib::translate::ToGlibPtr<'a, *const ffi::GstAudioFormatInfo> for AudioFormatInfo {
-    type Storage = &'a Self;
+unsafe impl glib::translate::TransparentPtrType for AudioFormatInfo {}
 
+#[doc(hidden)]
+impl<'a> glib::translate::ToGlibPtr<'a, *const ffi::GstAudioFormatInfo> for AudioFormatInfo {
+    type Storage = PhantomData<&'a Self>;
+
+    #[inline]
     fn to_glib_none(&'a self) -> glib::translate::Stash<'a, *const ffi::GstAudioFormatInfo, Self> {
-        glib::translate::Stash(self.0, self)
+        glib::translate::Stash(self.0, PhantomData)
     }
 
     fn to_glib_full(&self) -> *const ffi::GstAudioFormatInfo {
