@@ -1,16 +1,17 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+use std::{fmt, marker::PhantomData, mem, ptr, slice};
+
+use crate::ffi;
 use glib::translate::{
     from_glib, from_glib_full, from_glib_none, IntoGlib, ToGlibPtr, ToGlibPtrMut,
 };
 use gst::prelude::*;
 
-use std::fmt;
-use std::mem;
-use std::ptr;
-
 #[doc(alias = "GstAudioInfo")]
-pub struct AudioInfo(ffi::GstAudioInfo, [crate::AudioChannelPosition; 64]);
+#[derive(Clone)]
+#[repr(transparent)]
+pub struct AudioInfo(ffi::GstAudioInfo);
 
 impl fmt::Debug for AudioInfo {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -42,34 +43,23 @@ impl<'a> AudioInfoBuilder<'a> {
         unsafe {
             let mut info = mem::MaybeUninit::uninit();
 
-            let positions = if let Some(p) = self.positions {
+            if let Some(p) = self.positions {
                 if p.len() != self.channels as usize || p.len() > 64 {
                     return Err(glib::bool_error!("Invalid positions length"));
                 }
 
-                let positions: [ffi::GstAudioChannelPosition; 64] = std::array::from_fn(|i| {
-                    if i >= self.channels as usize {
-                        ffi::GST_AUDIO_CHANNEL_POSITION_INVALID
-                    } else {
-                        p[i].into_glib()
-                    }
-                });
-
                 let valid: bool = from_glib(ffi::gst_audio_check_valid_channel_positions(
-                    positions.as_ptr() as *mut _,
+                    p.as_ptr() as *mut _,
                     self.channels as i32,
                     true.into_glib(),
                 ));
                 if !valid {
                     return Err(glib::bool_error!("channel positions are invalid"));
                 }
+            }
 
-                Some(positions)
-            } else {
-                None
-            };
-
-            let positions_ptr = positions
+            let positions_ptr = self
+                .positions
                 .as_ref()
                 .map(|p| p.as_ptr())
                 .unwrap_or(ptr::null());
@@ -96,8 +86,7 @@ impl<'a> AudioInfoBuilder<'a> {
                 info.layout = layout.into_glib();
             }
 
-            let positions = std::array::from_fn(|i| from_glib(info.position[i]));
-            Ok(AudioInfo(info, positions))
+            Ok(AudioInfo(info))
         }
     }
 
@@ -108,6 +97,29 @@ impl<'a> AudioInfoBuilder<'a> {
         }
     }
 
+    pub fn positions_if(
+        self,
+        positions: &'a [crate::AudioChannelPosition],
+        predicate: bool,
+    ) -> AudioInfoBuilder<'a> {
+        if predicate {
+            self.positions(positions)
+        } else {
+            self
+        }
+    }
+
+    pub fn positions_if_some(
+        self,
+        positions: Option<&'a [crate::AudioChannelPosition]>,
+    ) -> AudioInfoBuilder<'a> {
+        if let Some(positions) = positions {
+            self.positions(positions)
+        } else {
+            self
+        }
+    }
+
     pub fn flags(self, flags: crate::AudioFlags) -> Self {
         Self {
             flags: Some(flags),
@@ -115,12 +127,44 @@ impl<'a> AudioInfoBuilder<'a> {
         }
     }
 
+    pub fn flags_if(self, flags: crate::AudioFlags, predicate: bool) -> Self {
+        if predicate {
+            self.flags(flags)
+        } else {
+            self
+        }
+    }
+
+    pub fn flags_if_some(self, flags: Option<crate::AudioFlags>) -> Self {
+        if let Some(flags) = flags {
+            self.flags(flags)
+        } else {
+            self
+        }
+    }
+
     pub fn layout(self, layout: crate::AudioLayout) -> Self {
         Self {
             layout: Some(layout),
             ..self
         }
     }
+
+    pub fn layout_if(self, layout: crate::AudioLayout, predicate: bool) -> Self {
+        if predicate {
+            self.layout(layout)
+        } else {
+            self
+        }
+    }
+
+    pub fn layout_if_some(self, layout: Option<crate::AudioLayout>) -> Self {
+        if let Some(layout) = layout {
+            self.layout(layout)
+        } else {
+            self
+        }
+    }
 }
 
 impl AudioInfo {
@@ -141,6 +185,7 @@ impl AudioInfo {
         }
     }
 
+    #[inline]
     pub fn is_valid(&self) -> bool {
         !self.0.finfo.is_null() && self.0.channels > 0 && self.0.rate > 0 && self.0.bpf > 0
     }
@@ -155,9 +200,7 @@ impl AudioInfo {
                 info.as_mut_ptr(),
                 caps.as_ptr(),
             )) {
-                let info = info.assume_init();
-                let positions = std::array::from_fn(|i| from_glib(info.position[i]));
-                Ok(Self(info, positions))
+                Ok(Self(info.assume_init()))
             } else {
                 Err(glib::bool_error!("Failed to create AudioInfo from caps"))
             }
@@ -222,6 +265,7 @@ impl AudioInfo {
         }
     }
 
+    #[inline]
     pub fn format(&self) -> crate::AudioFormat {
         if self.0.finfo.is_null() {
             return crate::AudioFormat::Unknown;
@@ -230,87 +274,104 @@ impl AudioInfo {
         unsafe { from_glib((*self.0.finfo).format) }
     }
 
+    #[inline]
     pub fn format_info(&self) -> crate::AudioFormatInfo {
         crate::AudioFormatInfo::from_format(self.format())
     }
 
+    #[inline]
     pub fn layout(&self) -> crate::AudioLayout {
         unsafe { from_glib(self.0.layout) }
     }
 
+    #[inline]
     pub fn flags(&self) -> crate::AudioFlags {
         unsafe { from_glib(self.0.flags) }
     }
 
+    #[inline]
     pub fn rate(&self) -> u32 {
         self.0.rate as u32
     }
 
+    #[inline]
     pub fn channels(&self) -> u32 {
         self.0.channels as u32
     }
 
+    #[inline]
     pub fn bpf(&self) -> u32 {
         self.0.bpf as u32
     }
 
+    #[inline]
     pub fn bps(&self) -> u32 {
-        (self.format_info().depth() as u32) >> 3
+        self.format_info().depth() >> 3
     }
 
+    #[inline]
     pub fn depth(&self) -> u32 {
         self.format_info().depth()
     }
 
+    #[inline]
     pub fn width(&self) -> u32 {
         self.format_info().width()
     }
 
+    #[inline]
     pub fn endianness(&self) -> crate::AudioEndianness {
         self.format_info().endianness()
     }
 
+    #[inline]
     pub fn is_big_endian(&self) -> bool {
         self.format_info().is_big_endian()
     }
 
+    #[inline]
     pub fn is_little_endian(&self) -> bool {
         self.format_info().is_little_endian()
     }
 
+    #[inline]
     pub fn is_float(&self) -> bool {
         self.format_info().is_float()
     }
 
+    #[inline]
     pub fn is_integer(&self) -> bool {
         self.format_info().is_integer()
     }
 
+    #[inline]
     pub fn is_signed(&self) -> bool {
         self.format_info().is_signed()
     }
 
+    #[inline]
     pub fn positions(&self) -> Option<&[crate::AudioChannelPosition]> {
         if self.0.channels > 64 || self.is_unpositioned() {
             return None;
         }
 
-        Some(&self.1[0..(self.0.channels as usize)])
+        Some(unsafe {
+            slice::from_raw_parts(
+                &self.0.position as *const i32 as *const crate::AudioChannelPosition,
+                self.0.channels as usize,
+            )
+        })
     }
 
+    #[inline]
     pub fn is_unpositioned(&self) -> bool {
         self.flags().contains(crate::AudioFlags::UNPOSITIONED)
     }
 }
 
-impl Clone for AudioInfo {
-    fn clone(&self) -> Self {
-        unsafe { Self(ptr::read(&self.0), self.1) }
-    }
-}
-
 impl PartialEq for AudioInfo {
     #[doc(alias = "gst_audio_info_is_equal")]
+    #[inline]
     fn eq(&self, other: &Self) -> bool {
         unsafe { from_glib(ffi::gst_audio_info_is_equal(&self.0, &other.0)) }
     }
@@ -322,6 +383,7 @@ unsafe impl Send for AudioInfo {}
 unsafe impl Sync for AudioInfo {}
 
 impl glib::types::StaticType for AudioInfo {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { glib::translate::from_glib(ffi::gst_audio_info_get_type()) }
     }
@@ -361,6 +423,14 @@ impl glib::value::ToValue for AudioInfo {
     }
 }
 
+#[doc(hidden)]
+impl From<AudioInfo> for glib::Value {
+    fn from(v: AudioInfo) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[doc(hidden)]
 impl glib::value::ToValueOptional for AudioInfo {
     fn to_value_optional(s: Option<&Self>) -> glib::Value {
@@ -378,6 +448,7 @@ impl glib::value::ToValueOptional for AudioInfo {
 
 #[doc(hidden)]
 impl glib::translate::Uninitialized for AudioInfo {
+    #[inline]
     unsafe fn uninitialized() -> Self {
         mem::zeroed()
     }
@@ -390,10 +461,11 @@ impl glib::translate::GlibPtrDefault for AudioInfo {
 
 #[doc(hidden)]
 impl<'a> glib::translate::ToGlibPtr<'a, *const ffi::GstAudioInfo> for AudioInfo {
-    type Storage = &'a Self;
+    type Storage = PhantomData<&'a Self>;
 
+    #[inline]
     fn to_glib_none(&'a self) -> glib::translate::Stash<'a, *const ffi::GstAudioInfo, Self> {
-        glib::translate::Stash(&self.0, self)
+        glib::translate::Stash(&self.0, PhantomData)
     }
 
     fn to_glib_full(&self) -> *const ffi::GstAudioInfo {
@@ -401,14 +473,19 @@ impl<'a> glib::translate::ToGlibPtr<'a, *const ffi::GstAudioInfo> for AudioInfo
     }
 }
 
+#[doc(hidden)]
+impl glib::translate::FromGlibPtrNone<*const ffi::GstAudioInfo> for AudioInfo {
+    #[inline]
+    unsafe fn from_glib_none(ptr: *const ffi::GstAudioInfo) -> Self {
+        Self(ptr::read(ptr))
+    }
+}
+
 #[doc(hidden)]
 impl glib::translate::FromGlibPtrNone<*mut ffi::GstAudioInfo> for AudioInfo {
     #[inline]
     unsafe fn from_glib_none(ptr: *mut ffi::GstAudioInfo) -> Self {
-        Self(
-            ptr::read(ptr),
-            std::array::from_fn(|i| from_glib((*ptr).position[i])),
-        )
+        Self(ptr::read(ptr))
     }
 }
 
