@@ -1,15 +1,11 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::translate::*;
+use std::{mem, ptr};
 
+use glib::translate::*;
 use gst::subclass::prelude::*;
 
-use std::mem;
-use std::ptr;
-
-use crate::prelude::*;
-
-use crate::AudioDecoder;
+use crate::{ffi, prelude::*, AudioDecoder};
 
 pub trait AudioDecoderImpl: AudioDecoderImplExt + ElementImpl {
     fn open(&self) -> Result<(), gst::ErrorMessage> {
@@ -90,52 +86,12 @@ pub trait AudioDecoderImpl: AudioDecoderImplExt + ElementImpl {
     }
 }
 
-pub trait AudioDecoderImplExt: ObjectSubclass {
-    fn parent_open(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_close(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_start(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_stop(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_set_format(&self, caps: &gst::Caps) -> Result<(), gst::LoggableError>;
-
-    fn parent_parse(&self, adapter: &gst_base::Adapter) -> Result<(u32, u32), gst::FlowError>;
-
-    fn parent_handle_frame(
-        &self,
-        buffer: Option<&gst::Buffer>,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_pre_push(&self, buffer: gst::Buffer) -> Result<Option<gst::Buffer>, gst::FlowError>;
-
-    fn parent_flush(&self, hard: bool);
-
-    fn parent_negotiate(&self) -> Result<(), gst::LoggableError>;
-
-    fn parent_caps(&self, filter: Option<&gst::Caps>) -> gst::Caps;
-
-    fn parent_sink_event(&self, event: gst::Event) -> bool;
-
-    fn parent_sink_query(&self, query: &mut gst::QueryRef) -> bool;
-
-    fn parent_src_event(&self, event: gst::Event) -> bool;
-
-    fn parent_src_query(&self, query: &mut gst::QueryRef) -> bool;
-
-    fn parent_propose_allocation(
-        &self,
-        query: &mut gst::query::Allocation,
-    ) -> Result<(), gst::LoggableError>;
-
-    fn parent_decide_allocation(
-        &self,
-        query: &mut gst::query::Allocation,
-    ) -> Result<(), gst::LoggableError>;
+mod sealed {
+    pub trait Sealed {}
+    impl<T: super::AudioDecoderImplExt> Sealed for T {}
 }
 
-impl<T: AudioDecoderImpl> AudioDecoderImplExt for T {
+pub trait AudioDecoderImplExt: sealed::Sealed + ObjectSubclass {
     fn parent_open(&self) -> Result<(), gst::ErrorMessage> {
         unsafe {
             let data = Self::type_data();
@@ -517,6 +473,8 @@ impl<T: AudioDecoderImpl> AudioDecoderImplExt for T {
     }
 }
 
+impl<T: AudioDecoderImpl> AudioDecoderImplExt for T {}
+
 unsafe impl<T: AudioDecoderImpl> IsSubclassable<T> for AudioDecoder {
     fn class_init(klass: &mut glib::Class<Self>) {
         Self::parent_class_init::<T>(klass);
@@ -644,8 +602,8 @@ unsafe extern "C" fn audio_decoder_parse<T: AudioDecoderImpl>(
     gst::panic_to_error!(imp, gst::FlowReturn::Error, {
         match imp.parse(&from_glib_borrow(adapter)) {
             Ok((new_offset, new_len)) => {
-                assert!(new_offset <= std::i32::MAX as u32);
-                assert!(new_len <= std::i32::MAX as u32);
+                assert!(new_offset <= i32::MAX as u32);
+                assert!(new_len <= i32::MAX as u32);
                 *offset = new_offset as i32;
                 *len = new_len as i32;
                 Ok(gst::FlowSuccess::Ok)
@@ -740,7 +698,7 @@ unsafe extern "C" fn audio_decoder_getcaps<T: AudioDecoderImpl>(
                 .as_ref(),
         )
     })
-    .to_glib_full()
+    .into_glib_ptr()
 }
 
 unsafe extern "C" fn audio_decoder_sink_event<T: AudioDecoderImpl>(
